use std::io::{BufReader, Read};

use borsh::{BorshDeserialize, BorshSerialize};
use xz2::bufread::XzEncoder;

#[derive(BorshSerialize, BorshDeserialize, PartialEq, Debug)]
enum Test {
    A(String),
}

#[derive(BorshSerialize, BorshDeserialize, PartialEq, Debug)]
struct A {
    x: u64,
    y: String,
    z: Test,
    uuids: Vec<u128>,
}

#[test]
fn test_simple_struct() {
    // let a = A {
    //     x: 3301,
    //     y: "liber primus".to_string(),
    //     z: Test::A("test".to_string()),
    // };
    // let encoded_a = a.try_to_vec().unwrap();
    // let decoded_a = A::try_from_slice(&encoded_a).unwrap();
    // assert_eq!(a, decoded_a);
}

fn main() {
    let mut uuid_vec: Vec<u128> = vec![];
    for i in 0..100 {
        uuid_vec.push(uuid::Uuid::new_v4().as_u128());
    }
    let a = A {
        x: 3301,
        y: "liber paasdafasdaa pastafari league cakffekokare robot The Internet is broken – and we are using it to access and distribute all of human knowledge. Data monopolies hold and enclose upon personal & public datasets, threatening universal access. To tackle this problem, between 2013 and 2015 we performed a series of design & development sprints in collaboration with researchers, librarians".to_string(),
        z: Test::A("test67d8c5d9-5237 The Internet is broken – and we are using it to access and distribute all of human knowledge. Data monopolies hold and enclose upon personal & public datasets, threatening universal access. To tackle this problem, between 2013 and 2015 we performed a series of design & development sprints in collaboration with researchers, librariansThe Internet is broken – and we are using it to access and distribute all of human knowledge. Data monopolies hold and enclose upon personal & public datasets, threatening universal access. To tackle this problem, between 2013 and 2015 we performed a series of design & development sprints in collaboration with researchers, librariansiInitially available only in English, versions in other languages were quickly developed. Wikipedia's combined editions comprise more than 61 million articles, attracting around 2 billion unique device visits per month and more than 15 million edits per month (about 5.8 edits per second on average) as of July 2023.[6][7]
Wikipedia".to_string()),
        uuids: uuid_vec,
    };
    let encoded_a = a.try_to_vec().unwrap();
    let decoded_a = A::try_from_slice(&encoded_a).unwrap();

    let mut bufread = BufReader::new(&encoded_a[..]);
    let mut compressed_lzma_rs = vec![];
    lzma_rs::lzma_compress(&mut bufread, &mut compressed_lzma_rs).unwrap();

    // let mut bufread = BufReader::new(&encoded_a[..]);
    // let mut compressed_lzma = lzma::compress(&mut bufread, 9).unwrap();

    let mut bufread = BufReader::new(&encoded_a[..]);
    let mut compressed_xz = vec![];
    XzEncoder::new(bufread, 9)
        .read_to_end(&mut compressed_xz)
        .unwrap();

    let mut bufread = BufReader::new(&encoded_a[..]);
    let compressed_zstd = zstd::encode_all(bufread, 9).unwrap();

    println!("{}", hex::encode(&encoded_a));
    println!();
    println!("{}", hex::encode(&compressed_lzma_rs));
    println!();
    println!("{}", hex::encode(&compressed_zstd));
    println!();
    println!("{}", hex::encode(&compressed_xz));

    println!("Size of encoded_a: {}", encoded_a.len());
    println!("Size of compressed_lzma_rs: {}", compressed_lzma_rs.len());
    println!("Size of compressed_zstd: {}", compressed_zstd.len());
    println!("Size of compressed_xz: {}", compressed_xz.len());
}
